static const char *RcsId = "$Header:  $";
//+=============================================================================
//
// file :         PhysicalValue.cpp
//
// description :  C++ source for the PhysicalValue and its commands. 
//                The class is derived from Device. It represents the
//                CORBA servant object which will be accessed from the
//                network. All commands which can be executed on the
//                PhysicalValue are implemented in this file.
//
// project :      TANGO Device Server
//
// $Author:  $
//
// $Revision:  $
//
// $Log:  $
//
// copyleft :    L.I.O.N.S
//				 Service de Chimie Molculaire
//				 Dpartement de Recherche sur l'Etat Condens, les Atomes et les Molcules
//				 Commissariat  L'Energie Atomique 
//               Centre d'Etudes de Saclay
//				 91191 Gif/Yvette Cedex
//               FRANCE
//				 http://www-drecam.cea.fr/scm/lions
//
//-=============================================================================
//
//  		This file is generated by POGO
//	(Program Obviously used to Generate tango Object)
//
//         (c) - Software Engineering Group - ESRF
//=============================================================================



//===================================================================
//
//	The folowing table gives the correspondance
//	between commands and method's name.
//
//  Command's name|  Method's name
//	----------------------------------------
//  State   |  dev_state()
//  Status  |  dev_status()
//
//===================================================================


#include <tango.h>
#include <PhysicalValue.h>
#include <PhysicalValueClass.h>

namespace PhysicalValue_ns
{

//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::PhysicalValue(string &s)
// 
// description : 	constructor for simulated PhysicalValue
//
// in : - cl : Pointer to the DeviceClass object
//      - s : Device name 
//
//-----------------------------------------------------------------------------
PhysicalValue::PhysicalValue(Tango::DeviceClass *cl,string &s)
:Tango::Device_3Impl(cl,s.c_str())
{
	init_device();
}

PhysicalValue::PhysicalValue(Tango::DeviceClass *cl,const char *s)
:Tango::Device_3Impl(cl,s)
{
	init_device();
}

PhysicalValue::PhysicalValue(Tango::DeviceClass *cl,const char *s,const char *d)
:Tango::Device_3Impl(cl,s,d)
{
	init_device();
}
//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::delete_device()
// 
// description : 	will be called at device destruction or at init command.
//
//-----------------------------------------------------------------------------
void PhysicalValue::delete_device()
{
	//	Delete device's allocated object
}

//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::init_device()
// 
// description : 	will be called at device initialization.
//
//-----------------------------------------------------------------------------
void PhysicalValue::init_device()
{
	INFO_STREAM << "PhysicalValue::PhysicalValue() create device " << device_name << endl;

	// Initialise variables to default values
	//--------------------------------------------
	get_device_property();

	attr_Value_read=new Tango::DevDouble;

	// ------ connection to attributes
	string proxy_name;
	try
	{
		if (attributeName_read!="")
		{
			
			proxy_name=attributeName_read;
			att_proxy_read = new Tango::AttributeProxy(attributeName_read);
			if (att_proxy_read == 0) throw std::bad_alloc();
			isReadable=true;
			cout << "connected to "<<proxy_name<<" for reading\n";
			
		}
		else
		{
			isReadable=false;
			cout << "NOT connected to "<<proxy_name<<"\n";
		}
		if (attributeName_write!="")
		{
			proxy_name=attributeName_write;
			att_proxy_write = new Tango::AttributeProxy(attributeName_write);
			if (att_proxy_write == 0) throw std::bad_alloc();
			isWritable=true;
			cout << "connected to "<<proxy_name<<" for writing\n";
		}
		else
		{
			isWritable=false;
			cout << "NOT connected to "<<proxy_name<<"\n";
		}
	}
  catch (const std::bad_alloc &)
  {
    ERROR_STREAM << "std::bad_alloc caught during AttributeProxy initialisation" << std::endl;
    set_status ("Initialization error [OUT_OF_MEMORY exception caught while initializing DeviceProxy]");
    set_state (Tango::FAULT);
	cout << "Initialization error [OUT_OF_MEMORY exception caught while initializing DeviceProxy : "<<proxy_name<<"]\n";
    return;
  }
	catch (Tango::ConnectionFailed &df)
  {
    ERROR_STREAM << (Tango::DevFailed&)df << std::endl;
    set_status ("Initialization error [TANGO_CONNECTION_FAILED caught while initializing DeviceProxy]");
    set_state (Tango::FAULT);
	cout <<"Initialization error [TANGO_CONNECTION_FAILED caught while initializing DeviceProxy: "<<proxy_name<<"]\n";
    return;
  }
	catch (Tango::WrongNameSyntax &df)
  {
    ERROR_STREAM << (Tango::DevFailed&)df << std::endl;
    set_status ("Initialization error [TANGO_WRONG_NAME_SYNTAX_ERROR caught while initializing DeviceProxy]");
    set_state (Tango::FAULT);
	cout << "Initialization error [TANGO_CONNECTION_FAILED caught while initializing DeviceProxy: "<<proxy_name<<"]\n";
    return;
  }
  

	
	/*Tango::AttributeInfo info=att_proxy_write->get_config();
	cout << info.writable<<"-"<<Tango::READ;
	*/

	

	if (variable_name=="")
	{
		cout<<"Variable Name properties is missing...\n";
		set_status ("Variable_name properties is missing...");
		set_state (Tango::FAULT);
		return;
	}
	
	if (expression_read=="")
	{
		cout<<"expression for reading properties is missing...\n";
		set_status ("expression_read for reading properties is missing...");
		set_state (Tango::FAULT);
		isReadable=false;
		return;
	}
	else
	{
		cout << variable_name<<" will be used for reading expression : "<<expression_read<<"\n";
	}
	if (expression_write=="")
	{
		cout<<"expression for writing properties is missing...\n";
		set_status ("expression_write for writing properties is missing...");
		set_state (Tango::FAULT);
		isWritable=false;
		return;
	}
	else
	{
		cout << variable_name<<" will be used for writing expression : "<<expression_write<<"\n";
	}
	
	// ----------  Parsing of expressions :

	if (isReadable)
	{

		char vr_read[500];
		char s_read[500];
		strcpy(vr_read,variable_name.c_str());
		xvar_read=new RVar( vr_read , &x_read );
		strcpy(s_read,expression_read.c_str());
		RVar* vararray_read[1]; vararray_read[0]=xvar_read;
		op_read=new ROperation( s_read, 1, vararray_read );
		if ((*op_read).HasError())
		{
			cout<<"expression "<<expression_read<< " is not valid...\n";
			set_status ("expression "+expression_read+" is not valid");
			set_state (Tango::FAULT);
			isReadable=false;
		}
	}
	if (isWritable)
	{
		char vr_write[500];
		char s_write[500];
		strcpy(vr_write,variable_name.c_str());
		xvar_write=new RVar( vr_write , &x_write );
		strcpy(s_write,expression_write.c_str());
		RVar* vararray_write[1]; vararray_write[0]=xvar_write;
		op_write=new ROperation( s_write, 1, vararray_write );
		if ((*op_write).HasError())
		{
			cout<<"expression "<<expression_write<< " is not valid...\n";
			set_status ("expression "+expression_write+" is not valid");
			set_state (Tango::FAULT);
			isWritable=false;
		}
	}
	
	
	
	

}


//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::get_device_property()
// 
// description : 	Read the device properties from database.
//
//-----------------------------------------------------------------------------
void PhysicalValue::get_device_property()
{
	//	Initialize your default values here (if not done with  POGO).
	//------------------------------------------------------------------

	//	Read device properties from database.(Automatic code generation)
	//------------------------------------------------------------------
	if (Tango::Util::instance()->_UseDb==false)
		return;
	Tango::DbData	dev_prop;
	dev_prop.push_back(Tango::DbDatum("AttributeName_read"));
	dev_prop.push_back(Tango::DbDatum("AttributeName_write"));
	dev_prop.push_back(Tango::DbDatum("Expression_read"));
	dev_prop.push_back(Tango::DbDatum("Expression_write"));
	dev_prop.push_back(Tango::DbDatum("Variable_name"));

	//	Call database and extract values
	//--------------------------------------------
	get_db_device()->get_property(dev_prop);
	Tango::DbDatum	def_prop, cl_prop;
	PhysicalValueClass	*ds_class =
		(static_cast<PhysicalValueClass *>(get_device_class()));
	int	i = -1;

	//	Try to initialize AttributeName_read from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  attributeName_read;
	//	Try to initialize AttributeName_read from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  attributeName_read;
	//	And try to extract AttributeName_read value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  attributeName_read;

	//	Try to initialize AttributeName_write from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  attributeName_write;
	//	Try to initialize AttributeName_write from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  attributeName_write;
	//	And try to extract AttributeName_write value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  attributeName_write;

	//	Try to initialize Expression_read from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  expression_read;
	//	Try to initialize Expression_read from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  expression_read;
	//	And try to extract Expression_read value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  expression_read;

	//	Try to initialize Expression_write from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  expression_write;
	//	Try to initialize Expression_write from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  expression_write;
	//	And try to extract Expression_write value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  expression_write;

	//	Try to initialize Variable_name from class property
	cl_prop = ds_class->get_class_property(dev_prop[++i].name);
	if (cl_prop.is_empty()==false)	cl_prop  >>  variable_name;
	//	Try to initialize Variable_name from default device value
	def_prop = ds_class->get_default_device_property(dev_prop[i].name);
	if (def_prop.is_empty()==false)	def_prop  >>  variable_name;
	//	And try to extract Variable_name value from database
	if (dev_prop[i].is_empty()==false)	dev_prop[i]  >>  variable_name;



	//	End of Automatic code generation
	//------------------------------------------------------------------

}

//+------------------------------------------------------------------
/**
 *	method:	PhysicalValue::dev_state
 *
 *	description:	method to execute "State"
 *	This command gets the device state (stored in its <i>device_state</i> data member) and returns it to the caller.
 *
 * @return	State Code
 *
 */
//+------------------------------------------------------------------
Tango::DevState PhysicalValue::dev_state()
{
	Tango::DevState	argout = DeviceImpl::dev_state();
	DEBUG_STREAM << "PhysicalValue::dev_state(): entering... !" << endl;

	//	Add your own code to control device here
	//Tango::DeviceProxy *my_device;
	//my_device=att_proxy_write->get_device_proxy();
	argout=att_proxy_read->state();
	//att_proxy_read = new Tango::AttributeProxy(device_Server_read+"/"+attributeName_read);
	if (argout==Tango::ON)
		argout=att_proxy_write->state();

	return argout;
}


//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::always_executed_hook()
// 
// description : 	method always executed before any command is executed
//
//-----------------------------------------------------------------------------
void PhysicalValue::always_executed_hook()
{
	
}
//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::read_attr_hardware
// 
// description : 	Hardware acquisition for attributes.
//
//-----------------------------------------------------------------------------
void PhysicalValue::read_attr_hardware(vector<long> &attr_list)
{
	DEBUG_STREAM << "PhysicalValue::read_attr_hardware(vector<long> &attr_list) entering... "<< endl;
	//	Add your own code here
}

//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::read_Value
// 
// description : 	Extract real attribute values for Value acquisition result.
//
//-----------------------------------------------------------------------------
void PhysicalValue::read_Value(Tango::Attribute &attr)
{
	DEBUG_STREAM << "PhysicalValue::read_Value(Tango::Attribute &attr) entering... "<< endl;
	Tango::DevDouble conversion;
	if (isReadable)
	{
		x_read=read_on_DeviceProxy();
		conversion=(*op_read).Val();
	}
	else
	{
		conversion=0.0;
	}
	*attr_Value_read = conversion;
	attr.set_value(attr_Value_read);
}

//+----------------------------------------------------------------------------
//
// method : 		PhysicalValue::write_Value
// 
// description : 	Write Value attribute values to hardware.
//
//-----------------------------------------------------------------------------
void PhysicalValue::write_Value(Tango::WAttribute &attr)
{
	DEBUG_STREAM << "PhysicalValue::write_Value(Tango::WAttribute &attr) entering... "<< endl;
	Tango::DevDouble value;
	attr.get_write_value(attr_Value_write);
	x_write=attr_Value_write;
	if (isWritable)
	{
		value=(*op_write).Val();
		write_on_DeviceProxy(value);
	}
	

}

//+------------------------------------------------------------------
/**
 *	method:	PhysicalValue::write_on_DeviceProxy()
 *
 *	description:	write a value on remote device server
 *	
 *	@argin : arg
 * @return	
 *
 */
//+------------------------------------------------------------------
void PhysicalValue::write_on_DeviceProxy(Tango::DevDouble arg)
{
	Tango::DevBoolean conv_Boolean;
	Tango::DevShort conv_Short;
	Tango::DevLong conv_Long;
	Tango::DevFloat conv_Float;
	Tango::DeviceAttribute da;
	cout << "will writing "<< arg<<"\n";
	da = att_proxy_write->read();
    /* TANGO DATA TYPE
	Type name 			Type code 			Value
	----------------------------------------------
	Tango::DevBoolean 	Tango::DEV_BOOLEAN 	1
	Tango::DevShort 	Tango::DEV_SHORT 	2
	Tango::DevLong 		Tango::DEV_LONG 	3
	Tango::DevFloat 	Tango::DEV_FLOAT 	4
	Tango::DevDouble 	Tango::DEV_DOUBLE 	5
	Tango::DevUShort 	Tango::DEV_USHORT 	6
	Tango::DevULong 	Tango::DEV_ULONG 	7
	Tango::DevString 	Tango::DEV_STRING 	8
	*/
	switch (da.get_type())
	{
	case 1:
		conv_Boolean=bool(arg);
		da<<conv_Boolean;
		break;
	case 2:
		conv_Short=int(arg);
		da<<conv_Short;
		break;
	case 3:
		conv_Long=long(arg);
		da<<conv_Long;
		break;
	case 4:
		conv_Float=float(arg);
		da<<conv_Float;
		break;
	case 5:
		da<<arg;//same type
		break;
	case 6:
		conv_Short=int(arg);
		da<<conv_Short;
		break;
	case 7:
		conv_Long=long(arg);
		da<<conv_Long;
		break;
	default :
		da<<arg;//same type
		break;
	}

	att_proxy_write->write(da);
	
}
//+------------------------------------------------------------------
/**
 *	method:	PhysicalValue::read_on_DeviceProxy()
 *
 *	description:	write a value on remote device server
 *	
 *	@argin : 
 * @return	value
 *
 */
//+------------------------------------------------------------------
Tango::DevDouble PhysicalValue::read_on_DeviceProxy()
{
	Tango::DevBoolean conv_Boolean;
	Tango::DevShort conv_Short;
	Tango::DevLong conv_Long;
	Tango::DevFloat conv_Float;
	Tango::DevDouble arg;
	Tango::DeviceAttribute da;

	da = att_proxy_read->read();
    /* TANGO DATA TYPE
	Type name 			Type code 			Value
	----------------------------------------------
	Tango::DevBoolean 	Tango::DEV_BOOLEAN 	1
	Tango::DevShort 	Tango::DEV_SHORT 	2
	Tango::DevLong 		Tango::DEV_LONG 	3
	Tango::DevFloat 	Tango::DEV_FLOAT 	4
	Tango::DevDouble 	Tango::DEV_DOUBLE 	5
	Tango::DevUShort 	Tango::DEV_USHORT 	6
	Tango::DevULong 	Tango::DEV_ULONG 	7
	Tango::DevString 	Tango::DEV_STRING 	8
	*/
	switch (da.get_type())
	{
	case 1:
		da>>conv_Boolean;
		arg=double(conv_Boolean);
		break;
	case 2:
		da>>conv_Short;
		arg=double(conv_Short);
		break;
	case 3:
		da>>conv_Long;
		arg=double(conv_Long);
		break;
	case 4:
		da>>conv_Float;
		arg=double(conv_Float);
		break;
	case 5:
		da>>arg;//same type
		break;
	case 6:
		da>>conv_Short;
		arg=double(conv_Short);
		break;
	case 7:
		da>>conv_Long;
		arg=double(conv_Long);
		break;
	default :
		da>>arg;//same type
		break;
	}

	return arg;
	
}

}	//	namespace
